package com.example.bigsql.service;


import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.thread.ExecutorBuilder;
import com.example.bigsql.util.JyDbUtil;

import java.io.BufferedReader;
import java.io.FileReader;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * 解析v3版大sql文件
 */

public class ImportBigSqlV3File {
    static AtomicInteger isSaveSize = new AtomicInteger(0);
    static AtomicInteger totalSize = new AtomicInteger(0);

    public static void main(String[] args) {
        String filePath = args[0];
        String databaseName = args[1];
        System.out.println("单读取开始时间：" + DateUtil.now());
        List<String> dataLinkList = Collections.synchronizedList(new ArrayList<>());
        int batchSize = Convert.toInt(args[2], 20000);


        try (BufferedReader reader = new BufferedReader(new FileReader(filePath))) {

            ExecutorService executor = ExecutorBuilder.create()//
                    .setCorePoolSize(20)//
                    .setMaxPoolSize(40)//
                    .setKeepAliveTime(0)//
                    .build();
            Connection conn = JyDbUtil.conn;
            conn.setAutoCommit(false);
            String line;
            while ((line = reader.readLine()) != null) {

                if (line.startsWith("INSERT")) {
                    dataLinkList.add(line);

                    if (dataLinkList.size() == batchSize) {
                        System.out.println("=========当前线程名==========" + Thread.currentThread().getName() + "===================");
                        System.out.println("插入开始时间：" + DateUtil.now());

                        Statement statement = conn.createStatement();
                        System.out.println("创建数据库链接：" + DateUtil.now());
                        totalSize.set(totalSize.get() + batchSize);

                        System.out.println("总数累计：" + DateUtil.now());
                        List<String> copyFruits = dataLinkList.stream().collect(Collectors.toList());
                        System.out.println("复制list完成时间：" + DateUtil.now());
                        System.out.println("读取插入语句有" + batchSize + "条，执行批量插入新线程");

                        executor.submit(new Runnable() {
                            //获得当前线程的名称

                            @Override
                            public void run() {
                                try {
                                    copyFruits.forEach(sql -> {
                                        try {
                                            sql = sql.replace("INTO `", "INTO `" + databaseName + "`.`").replace(";", "");
                                            statement.addBatch(sql);
                                        } catch (SQLException ex) {
                                            throw new RuntimeException(ex);
                                        }
                                    });

                                    int[] i = statement.executeBatch();
                                    conn.commit();
                                    System.out.println("=========当前线程名==========" + Thread.currentThread().getName() + "===================");
                                    isSaveSize.set(isSaveSize.get() + i.length);
                                    System.out.println("已读取：" + totalSize.get() + "已插入：" + isSaveSize.get() + " 完成：" + bs(isSaveSize.get(), totalSize.get()) + "%");
                                } catch (Exception e) {
                                    throw new RuntimeException(e);
                                } finally {
                                    try {
                                        copyFruits.clear();
                                        statement.clearBatch();
                                        statement.close();
                                    } catch (SQLException e) {
                                        throw new RuntimeException(e);
                                    }
                                }
                            }
                        });
                        dataLinkList.clear();
                    }

                }
            }

        } catch (
                Exception e) {
            throw new RuntimeException("Failed to read SQL file: " + filePath, e);
        }
    }

    public static int bs(int a, int b) {
        return (int) ((new BigDecimal((float) a / b).setScale(2, RoundingMode.HALF_UP).doubleValue()) * 100);
    }

}
