package com.example.bigsql.service;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.example.bigsql.util.JyDbUtil;
import lombok.extern.slf4j.Slf4j;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
public class JyDealBigSqlFile {
    public static void main(String[] args) {
        log.info("开始处理时间：{}", DateUtil.now());
        String stationCode = "J00300";
        //1.读文件，生成数据库。分割文件
        String filePath = "E:\\迅雷下载\\ipfcst-vv3.sql";
        String csvFilePath = "E:\\迅雷下载\\csvdir\\";
        Map<String, FileWriter> fileWriterMap = new HashMap<>();
        try (BufferedReader reader = new BufferedReader(new FileReader(filePath))) {
            List<String> sqlStatements = new ArrayList<>();
            StringBuilder statementBuilder = new StringBuilder();
            String line;
            int lineSize = 0;
            while ((line = reader.readLine()) != null) {

                if (line.startsWith("Navicat") || line.startsWith("/")
                        || line.startsWith("INSERT") || line.startsWith("--") || line.startsWith("#")
                        || line.startsWith("//")) {
                    lineSize++;
                    spliteFileToCsv(line, fileWriterMap, csvFilePath);
                    continue; // Skip comments
                }
                if (line.endsWith(";")) {
                    statementBuilder.append(line); // Remove semicolon
                    sqlStatements.add(statementBuilder.toString().trim()); // Add to list
                    statementBuilder.setLength(0); // Reset statement buffer
                } else {
                    statementBuilder.append(line).append(" ");
                }
            }
            sqlStatements = sqlStatements.stream().filter(s -> s.startsWith("CREATE") || s.startsWith("DROP")).collect(Collectors.toList());

            String databasesName = "ipfcst_" + stationCode + "_" + DateUtil.format(new Date(), DatePattern.PURE_DATETIME_FORMAT);

            Connection conn = JyDbUtil.conn;
            if (createDatabase(databasesName, conn)) {
                if (createTable(databasesName, sqlStatements, conn)) {
                    log.info("执行数据库和数据表创建成功,其中数据共：" + lineSize);
                }
            }
        } catch (IOException e) {
            throw new RuntimeException("Failed to read SQL file: " + filePath, e);
        }

        log.info("完成处理时间：{}", DateUtil.now());
    }


    //把insert语句分割成csv文件
    public static void spliteFileToCsv(String line, Map<String, FileWriter> fileWriterMap, String csvFilePath) {
        String outputFilePath = ".csv";
        try {
            if (line.trim().startsWith("INSERT")) {
                String tableName = line.substring(13, line.indexOf("` VALUES"));
                if (null == fileWriterMap.get(tableName)) {
                    fileWriterMap.put(tableName, new FileWriter(csvFilePath + tableName + outputFilePath));
                }
                // 解析SQL语句，提取数据
                String[] data = parseSQLLine(line);

                if (data != null) {
                    // 将数据以CSV格式写入文件
                    fileWriterMap.get(tableName).write(String.join(",", data));
                    fileWriterMap.get(tableName).write("\n");
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    // 解析SQL语句中的数据
    private static String[] parseSQLLine(String line) {
        if (line.trim().startsWith("INSERT")) {
            //解析INSERT语句中的数据
            int startIndex = line.indexOf("(");
            int endIndex = line.indexOf(")");
            if (startIndex == -1 || endIndex == -1) {
                return null;
            }
            String data = line.substring(startIndex + 1, endIndex).replaceAll("'", "");

            return data.replace(", ", ",").replace("\\", "").split(",");
        }
        return null;

    }


    //创建数据库
    public static boolean createDatabase(String tableName, Connection conn) {
        try {
            Statement statement = conn.createStatement();
            statement.execute("CREATE DATABASE `" + tableName + "` CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci");

            log.info(String.format("Database '%s' created successfully!", tableName));
            return true;
        } catch (Exception e) {
            log.info(String.format("Database '%s' created error!", tableName));
        }
        return false;
    }

    //创建各个表
    private static boolean createTable(String databasesName, List<String> sqlStatements, Connection conn) {
        try {
            conn.setAutoCommit(false);
            Statement statement = conn.createStatement();
            sqlStatements.forEach(sql -> {
                sql = sql.replace("EXISTS `", "EXISTS `" + databasesName + "`.`").replace(";", "");
                sql = sql.replace("TABLE `", "TABLE `" + databasesName + "`.`").replace(";", "");
                try {
                    statement.addBatch(sql);

                } catch (SQLException e) {
                    throw new RuntimeException(e);
                }
            });
            int[] i = statement.executeBatch();
            conn.commit();
            log.info("成功执行语句：" + i.length);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }
}
