package com.example.bigsql.util;

import com.github.junrar.Archive;
import com.github.junrar.rarfile.FileHeader;

import java.io.*;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.zip.GZIPInputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

public class UnZipUtil {
    //解压.zip文件
    public static void unZip(String sourceFile, String outputDir) throws IOException {
        ZipFile zipFile = null;
        File file = new File(sourceFile);
        try {
            Charset CP866 = StandardCharsets.UTF_8;  //specifying alternative (non UTF-8) charset
            zipFile = new ZipFile(file, Charset.forName("GBK"));
            createDirectory(outputDir, null);//创建输出目录

            Enumeration<?> enums = zipFile.entries();
            while (enums.hasMoreElements()) {

                ZipEntry entry = (ZipEntry) enums.nextElement();
                System.out.println("解压." + entry.getName());

                if (entry.isDirectory()) {//是目录
                    createDirectory(outputDir, entry.getName());//创建空目录
                } else {//是文件
                    File tmpFile = new File(outputDir + "/" + entry.getName());
                    createDirectory(tmpFile.getParent() + "/", null);//创建输出目录

                    try (InputStream in = zipFile.getInputStream(entry); OutputStream out = Files.newOutputStream(tmpFile.toPath())) {
                        int length = 0;

                        byte[] b = new byte[2048];
                        while ((length = in.read(b)) != -1) {
                            out.write(b, 0, length);
                        }

                    }
                }
            }

        } catch (IOException e) {
            throw new IOException("解压缩文件出现异常", e);
        } finally {
            try {
                if (zipFile != null) {
                    zipFile.close();
                }
            } catch (IOException ex) {
                throw new IOException("关闭zipFile出现异常", ex);
            }
        }
    }

    public static void unzip1(File file, String outputDir) throws IOException {
        File outFile = null;   // 输出文件的时候要有文件夹的操作
        ZipFile zipFile = new ZipFile(file.getPath());   // 实例化ZipFile对象
        ZipInputStream zipInput = null;    // 定义压缩输入流

        //定义解压的文件名
        OutputStream out = null;   // 定义输出流，用于输出每一个实体内容
        InputStream input = null;  // 定义输入流，读取每一个ZipEntry
        ZipEntry entry = null; // 每一个压缩实体
        zipInput = new ZipInputStream(new FileInputStream(file));  // 实例化ZIpInputStream

        //遍历压缩包中的文件
        while ((entry = zipInput.getNextEntry()) != null) { // 得到一个压缩实体
            System.out.println("解压缩" + entry.getName() + "文件");
            outFile = new File(outputDir + File.separator + entry.getName());   // 定义输出的文件路径
            if (!outFile.getParentFile().exists()) {  // 如果输出文件夹不存在
                outFile.getParentFile().mkdirs();
                // 创建文件夹 ,如果这里的有多级文件夹不存在,请使用mkdirs()
                // 如果只是单纯的一级文件夹,使用mkdir()就好了
            }
            if (!outFile.exists()) {  // 判断输出文件是否存在
                if (entry.isDirectory()) {
                    outFile.mkdirs();
                    System.out.println("create directory...");
                } else {
                    outFile.createNewFile();   // 创建文件
                    System.out.println("create file...");
                }
            }
            if (!entry.isDirectory()) {
                input = zipFile.getInputStream(entry); // 得到每一个实体的输入流
                out = new FileOutputStream(outFile);   // 实例化文件输出流
                int temp = 0;
                while ((temp = input.read()) != -1) {
                    out.write(temp);
                }
                input.close();     // 关闭输入流
                out.close();   // 关闭输出流
            }

        }
        input.close();
    }

    /**
     * 构建目录
     *
     * @param outputDir
     * @param subDir
     */
    public static void createDirectory(String outputDir, String subDir) {
        File file = new File(outputDir);
        if (!(subDir == null || subDir.trim().equals(""))) {//子目录不为空
            file = new File(outputDir + "/" + subDir);
        }
        if (!file.exists()) {
            if (!file.getParentFile().exists())
                file.getParentFile().mkdirs();
            file.mkdirs();
        }
    }


    //解压.rar文件
    public static void unRar(String sourceFile, String outputDir) throws Exception {
        Archive archive = null;
        FileOutputStream fos = null;
        File file = new File(sourceFile);
        try {
            archive = new Archive(file);
            FileHeader fh = archive.nextFileHeader();
            int count = 0;
            File destFileName = null;
            while (fh != null) {
                System.out.println((++count) + ") " + fh.getFileNameString());
                String compressFileName = fh.getFileNameString().trim();
                destFileName = new File(outputDir + "/" + compressFileName);
                if (fh.isDirectory()) {
                    if (!destFileName.exists()) {
                        destFileName.mkdirs();
                    }
                    fh = archive.nextFileHeader();
                    continue;
                }
                if (!destFileName.getParentFile().exists()) {
                    destFileName.getParentFile().mkdirs();
                }
                fos = new FileOutputStream(destFileName);
                archive.extractFile(fh, fos);
                fos.close();
                fos = null;
                fh = archive.nextFileHeader();
            }

            archive.close();
            archive = null;
        } catch (Exception e) {
            throw e;
        } finally {
            if (fos != null) {
                try {
                    fos.close();
                    fos = null;
                } catch (Exception e) {
                    //ignore
                }
            }
            if (archive != null) {
                try {
                    archive.close();
                    archive = null;
                } catch (Exception e) {
                    //ignore
                }
            }
        }
    }


    //解压.gz文件
    public static void unGz(String sourceFile, String outputDir) {
        String ouputfile = "";
        try {
            //建立gzip压缩文件输入流
            FileInputStream fin = new FileInputStream(sourceFile);
            //建立gzip解压工作流
            GZIPInputStream gzin = new GZIPInputStream(fin);
            //建立解压文件输出流
            /*ouputfile = sourceFile.substring(0,sourceFile.lastIndexOf('.'));
            ouputfile = ouputfile.substring(0,ouputfile.lastIndexOf('.'));*/
            File file = new File(sourceFile);
            String fileName = file.getName();
            outputDir = outputDir + "/" + fileName.substring(0, fileName.lastIndexOf('.'));
            FileOutputStream fout = new FileOutputStream(outputDir);

            int num;
            byte[] buf = new byte[1024];

            while ((num = gzin.read(buf, 0, buf.length)) != -1) {
                fout.write(buf, 0, num);
            }

            gzin.close();
            fout.close();
            fin.close();
        } catch (Exception ex) {
            System.err.println(ex);
        }
    }

    /*public static void main(String[] args) throws IOException {
        ZipFile zipFile = new ZipFile("/Users/xiaowang/Downloads/J00307.zip");
    }*/
    public static void main(String[] args) throws IOException, SQLException {
        //连接mysql
        Connection conn = null;
        PreparedStatement ps = null;

        try {
            String url = "jdbc:mysql://192.168.1.119:14000/jiayueCloud_local?useUnicode=true&characterEncoding=utf8&zeroDateTimeBehavior=convertToNull&useSSL=true&serverTimezone=GMT%2B8&autoReconnect=true&rewriteBatchedStatements=true";
            //加载数据库驱动
            Class.forName("com.mysql.cj.jdbc.Driver");
            conn = DriverManager.getConnection(url, "root", "M*R7Dp+H4ey2Y^319@");
            //String sql = "UPDATE dc_upload_bigsql_file_record set status = '" + status + "' , db_name = '" + dbName + "' WHERE id = " + id + ";";
            String sql = "UPDATE dc_upload_bigsql_file_record set status = '" + "1" + "' , db_name = '" + "ipfcst_J00307_20202" + "' WHERE id = " + 30001 + ";";
            ps = conn.prepareStatement(sql);
            ps.executeUpdate();
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            if (ps != null) {
                ps.close();
            }
            if (conn != null) {
                conn.close();
            }
        }
    }

}
