package com.example.datadump.controller;

import com.example.datadump.service.DumpMysql;
import com.example.datadump.service.DumpTDEngine;
import com.example.datadump.service.ImportMysql;
import com.example.datadump.service.ImportTDEngine;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@AllArgsConstructor
@Slf4j
public class DumpDataController {
    private final DumpMysql dumpMysql;
    private final DumpTDEngine dumpTDEngine;
    private final ImportMysql importMysql;
    private final ImportTDEngine importTDEngine;

    /**
     * 备份mysql数据库
     *
     * @param userName 用户名
     * @param passWord 密码
     * @param ip       IP地址
     * @param port     端口
     * @param db       数据库
     * @param table    表
     * @param outPath  输出路径
     * @throws Exception
     */
    @GetMapping("/DumpMysql")
    public void DumpMysql(String userName, String passWord, String ip, String port, String db, String table, String outPath) throws Exception {
        log.info("MySQL数据备份开始");
        dumpMysql.dbBackUpMysql(userName, passWord, ip, port, db, table, outPath);
        log.info("MySQL数据备份结束");
    }

    /**
     * 恢复mysql数据库
     *
     * @param userName 用户名
     * @param passWord 密码
     * @param ip       IP地址
     * @param port     端口
     * @param db       数据库
     * @param table    表
     * @param outPath  输出路径
     * @throws Exception
     */
    @GetMapping("/ImportMysql")
    public void ImportMysql(String userName, String passWord, String ip, String port, String db, String table, String outPath) throws Exception {
        log.info("MySQL数据恢复开始");
        importMysql.importMysql(userName, passWord, ip, port, db, table, outPath);
        log.info("MySQL数据恢复结束");
    }

    /**
     * TDEngine数据库备份
     *
     * @param ip       IP地址
     * @param port     端口号
     * @param db       数据库
     * @param userName 用户名
     * @param passWord 密码
     * @param table    表名,如果多个表以逗号分隔
     * @param path     导出路径
     * @throws Exception
     */
    @GetMapping("/DumpTDEngine")
    public void DumpTEngine(String ip, String port, String db, String userName, String passWord, String table, String path) throws Exception {
        log.info("TDEngine数据备份开始");
        dumpTDEngine.dbBackUpTDEngine(ip, port, db, userName, passWord, table, path);
        log.info("TDEngine数据备份结束");
    }

    @GetMapping("/ImportTDEngine")
    public void ImportTDEngine(String ip, String port, String db, String userName, String passWord, String table, String path) throws Exception {
        log.info("TDEngine数据恢复开始");
        importTDEngine.importTDEngine(ip, port, db, userName, passWord, table, path);
        log.info("TDEngine数据恢复结束");
    }
}
